"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _SolanaTransport_conn, _SolanaTransport_legacyTokens;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaTransport = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const sdk_core_1 = require("@race-foundation/sdk-core");
const instruction = __importStar(require("./instruction"));
const constants_1 = require("./constants");
const accounts_1 = require("./accounts");
const instruction_1 = require("./instruction");
const constants_2 = require("./constants");
const metadata_1 = require("./metadata");
function trimString(s) {
    return s.replace(/\0/g, '');
}
class SolanaTransport {
    constructor(endpoint) {
        _SolanaTransport_conn.set(this, void 0);
        _SolanaTransport_legacyTokens.set(this, void 0);
        __classPrivateFieldSet(this, _SolanaTransport_conn, new web3_js_1.Connection(endpoint, 'confirmed'), "f");
    }
    async _fetchLegacyTokens() {
        const resp = await fetch("https://cdn.jsdelivr.net/gh/solflare-wallet/token-list/solana-tokenlist.json");
        const m = await resp.json();
        __classPrivateFieldSet(this, _SolanaTransport_legacyTokens, m['tokens'], "f");
    }
    async createGameAccount(wallet, params) {
        const { title, bundleAddr, tokenAddr } = params;
        if (title.length > constants_1.NAME_LEN) {
            // FIXME: better error message?
            throw Error('Game title length exceeds 16 chars');
        }
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const payerKey = new web3_js_1.PublicKey(wallet.walletAddr);
        console.log('Payer publick key: ', payerKey);
        let tx = await makeTransaction(conn, payerKey);
        const gameAccount = web3_js_1.Keypair.generate();
        const gameAccountKey = gameAccount.publicKey;
        const lamports = await conn.getMinimumBalanceForRentExemption(constants_1.GAME_ACCOUNT_LEN);
        const createGameAccount = web3_js_1.SystemProgram.createAccount({
            fromPubkey: payerKey,
            newAccountPubkey: gameAccountKey,
            lamports: lamports,
            space: constants_1.GAME_ACCOUNT_LEN,
            programId: constants_2.PROGRAM_ID,
        });
        tx.add(createGameAccount);
        const tokenMintKey = new web3_js_1.PublicKey(tokenAddr);
        const stakeAccount = web3_js_1.Keypair.generate();
        const stakeAccountKey = stakeAccount.publicKey;
        const stakeLamports = await conn.getMinimumBalanceForRentExemption(spl_token_1.AccountLayout.span);
        const createStakeAccount = web3_js_1.SystemProgram.createAccount({
            fromPubkey: payerKey,
            newAccountPubkey: stakeAccountKey,
            lamports: stakeLamports,
            space: spl_token_1.AccountLayout.span,
            programId: spl_token_1.TOKEN_PROGRAM_ID,
        });
        tx.add(createStakeAccount);
        const initStakeAccount = (0, spl_token_1.createInitializeAccountInstruction)(stakeAccountKey, tokenMintKey, payerKey, spl_token_1.TOKEN_PROGRAM_ID);
        tx.add(initStakeAccount);
        const bundleKey = new web3_js_1.PublicKey(bundleAddr);
        const createGame = instruction.createGameAccount({
            ownerKey: payerKey,
            gameAccountKey: gameAccountKey,
            stakeAccountKey: stakeAccountKey,
            mint: tokenMintKey,
            gameBundleKey: bundleKey,
            title: title,
            maxPlayers: params.maxPlayers,
            minDeposit: params.minDeposit,
            maxDeposit: params.maxDeposit,
        });
        tx.add(createGame);
        tx.partialSign(gameAccount, stakeAccount);
        await wallet.sendTransaction(tx, conn);
        return gameAccountKey.toBase58();
    }
    async closeGameAccount(wallet, params) { }
    async join(wallet, params) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const { gameAddr, amount: amountRaw, accessVersion: accessVersionRaw, position, verifyKey } = params;
        const accessVersion = BigInt(accessVersionRaw);
        const playerKey = new web3_js_1.PublicKey(wallet.walletAddr);
        const gameAccountKey = new web3_js_1.PublicKey(gameAddr);
        const gameState = await this._getGameState(gameAccountKey);
        if (gameState === undefined) {
            throw new Error('TS: Game account not found');
        }
        const mintKey = gameState.tokenKey;
        const isWsol = mintKey.equals(spl_token_1.NATIVE_MINT);
        const amount = BigInt(amountRaw);
        if (amount < gameState.minDeposit || amount > gameState.maxDeposit) {
            console.log('Max deposit = {}, min deposit = {}, join amount = {}', gameState.maxDeposit, gameState.minDeposit, amount);
            throw new Error('TS: Join with invalid amount');
        }
        const stakeAccountKey = gameState.stakeKey;
        const tempAccountKeypair = web3_js_1.Keypair.generate();
        const tempAccountKey = tempAccountKeypair.publicKey;
        const tempAccountLen = spl_token_1.AccountLayout.span;
        const tempAccountLamports = await conn.getMinimumBalanceForRentExemption(tempAccountLen);
        const tx = await makeTransaction(conn, playerKey);
        const createTempAccountIx = web3_js_1.SystemProgram.createAccount({
            fromPubkey: playerKey,
            newAccountPubkey: tempAccountKey,
            lamports: tempAccountLamports,
            space: tempAccountLen,
            programId: spl_token_1.TOKEN_PROGRAM_ID,
        });
        tx.add(createTempAccountIx);
        const initTempAccountIx = (0, spl_token_1.createInitializeAccountInstruction)(tempAccountKey, mintKey, playerKey);
        tx.add(initTempAccountIx);
        if (isWsol) {
            const transferAmount = amount - BigInt(tempAccountLamports);
            const transferIx = web3_js_1.SystemProgram.transfer({
                fromPubkey: playerKey,
                toPubkey: tempAccountKey,
                lamports: transferAmount,
            });
            tx.add(transferIx);
        }
        else {
            const playerAta = (0, spl_token_1.getAssociatedTokenAddressSync)(mintKey, playerKey);
            const transferIx = (0, spl_token_1.createTransferInstruction)(playerAta, tempAccountKey, playerKey, amount);
            tx.add(transferIx);
        }
        const joinGameIx = await (0, instruction_1.join)({
            playerKey,
            paymentKey: tempAccountKey,
            gameAccountKey,
            mint: mintKey,
            stakeAccountKey: stakeAccountKey,
            amount,
            accessVersion,
            position,
            verifyKey
        });
        tx.add(joinGameIx);
        tx.partialSign(tempAccountKeypair);
        await wallet.sendTransaction(tx, __classPrivateFieldGet(this, _SolanaTransport_conn, "f"));
    }
    async deposit(wallet, params) { }
    async publishGame(wallet, params) {
        return '';
    }
    async vote(wallet, params) { }
    async createPlayerProfile(wallet, params) {
        const { nick, pfp } = params;
        if (nick.length > 16) {
            // FIXME: better error message?
            throw new Error('Player nick name exceeds 16 chars');
        }
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const payerKey = new web3_js_1.PublicKey(wallet.walletAddr);
        console.log('Payer Public Key:', payerKey);
        const profileKey = await web3_js_1.PublicKey.createWithSeed(payerKey, constants_1.PLAYER_PROFILE_SEED, constants_2.PROGRAM_ID);
        console.log('Player profile public key: ', profileKey);
        let tx = new web3_js_1.Transaction();
        if (!(await conn.getAccountInfo(profileKey))) {
            let lamports = await conn.getMinimumBalanceForRentExemption(constants_1.PROFILE_ACCOUNT_LEN);
            const createProfileAccount = web3_js_1.SystemProgram.createAccountWithSeed({
                fromPubkey: payerKey,
                newAccountPubkey: profileKey,
                basePubkey: payerKey,
                seed: constants_1.PLAYER_PROFILE_SEED,
                lamports: lamports,
                space: constants_1.PROFILE_ACCOUNT_LEN,
                programId: constants_2.PROGRAM_ID,
            });
            tx.add(createProfileAccount);
        }
        const pfpKey = !pfp ? web3_js_1.PublicKey.default : new web3_js_1.PublicKey(pfp);
        const createProfile = instruction.createPlayerProfile(payerKey, profileKey, nick, pfpKey);
        tx.add(createProfile);
        await wallet.sendTransaction(tx, conn);
    }
    async createRegistration(wallet, params) {
        return '';
    }
    async registerGame(wallet, params) { }
    async unregisterGame(wallet, params) { }
    async getGameAccount(addr) {
        const gameAccountKey = new web3_js_1.PublicKey(addr);
        const gameState = await this._getGameState(gameAccountKey);
        if (gameState !== undefined) {
            return gameState.generalize(new web3_js_1.PublicKey(addr));
        }
        else {
            return undefined;
        }
    }
    async getGameBundle(addr) {
        const mintKey = new web3_js_1.PublicKey(addr);
        console.log('Game Stake Mint:', mintKey);
        const [metadataKey] = web3_js_1.PublicKey.findProgramAddressSync([Buffer.from('metadata', 'utf8'), constants_2.METAPLEX_PROGRAM_ID.toBuffer(), mintKey.toBuffer()], constants_2.METAPLEX_PROGRAM_ID);
        const metadataAccount = await __classPrivateFieldGet(this, _SolanaTransport_conn, "f").getAccountInfo(metadataKey);
        if (metadataAccount === null) {
            return undefined;
        }
        const metadataState = metadata_1.Metadata.deserialize(metadataAccount.data);
        console.log('Metadata state:', metadataState);
        let { uri, name } = metadataState.data;
        return new sdk_core_1.GameBundle({ uri: trimString(uri), name: trimString(name), data: new Uint8Array(0) });
    }
    async getPlayerProfile(addr) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const playerKey = new web3_js_1.PublicKey(addr);
        const profileKey = await web3_js_1.PublicKey.createWithSeed(playerKey, constants_1.PLAYER_PROFILE_SEED, constants_2.PROGRAM_ID);
        const profileAccount = await conn.getAccountInfo(profileKey);
        if (profileAccount) {
            const profileAccountData = profileAccount.data;
            const state = accounts_1.PlayerState.deserialize(profileAccountData);
            return state.generalize(playerKey);
        }
        else {
            return undefined;
        }
    }
    async getServerAccount(addr) {
        const serverKey = new web3_js_1.PublicKey(addr);
        const serverState = await this._getServerState(serverKey);
        if (serverState !== undefined) {
            return serverState.generalize();
        }
        else {
            return undefined;
        }
    }
    async getRegistration(addr) {
        const regKey = new web3_js_1.PublicKey(addr);
        const regState = await this._getRegState(regKey);
        if (regState !== undefined) {
            return regState.generalize(regKey);
        }
        else {
            return undefined;
        }
    }
    async getRegistrationWithGames(addr) {
        const regAccount = await this.getRegistration(addr);
        if (regAccount === undefined)
            return undefined;
        const keys = regAccount.games.map(g => new web3_js_1.PublicKey(g.addr));
        const gameStates = await this._getMultiGameStates(keys);
        let games = [];
        for (let i = 0; i < gameStates.length; i++) {
            const gs = gameStates[i];
            if (gs === undefined) {
                games.push(undefined);
            }
            else {
                games.push(gs.generalize(keys[i]));
            }
        }
        return new sdk_core_1.RegistrationWithGames({
            ...regAccount,
            games
        });
    }
    async _fetchImageFromDataUri(dataUri) {
        try {
            const resp = await fetch(dataUri);
            const data = await resp.json();
            return data.image;
        }
        catch (e) {
            return undefined;
        }
    }
    async getToken(addr) {
        const mintKey = new web3_js_1.PublicKey(addr);
        try {
            const mint = await (0, spl_token_1.getMint)(__classPrivateFieldGet(this, _SolanaTransport_conn, "f"), mintKey, 'finalized');
            const [metadataKey] = web3_js_1.PublicKey.findProgramAddressSync([Buffer.from('metadata', 'utf8'), constants_2.METAPLEX_PROGRAM_ID.toBuffer(), mintKey.toBuffer()], constants_2.METAPLEX_PROGRAM_ID);
            const metadataAccount = await __classPrivateFieldGet(this, _SolanaTransport_conn, "f").getAccountInfo(metadataKey);
            let metadataState;
            if (metadataAccount !== null) {
                metadataState = metadata_1.Metadata.deserialize(metadataAccount.data);
            }
            // Get from legacy token
            if (__classPrivateFieldGet(this, _SolanaTransport_legacyTokens, "f") === undefined) {
                await this._fetchLegacyTokens();
            }
            let legacyToken = undefined;
            if (__classPrivateFieldGet(this, _SolanaTransport_legacyTokens, "f") !== undefined) {
                legacyToken = __classPrivateFieldGet(this, _SolanaTransport_legacyTokens, "f").find(t => t.address === addr);
            }
            if (metadataState !== undefined) {
                const addr = mint.address.toBase58();
                const decimals = mint.decimals;
                const image = await this._fetchImageFromDataUri(metadataState.data.uri);
                const name = metadataState.data.name ? trimString(metadataState.data.name) :
                    legacyToken ? legacyToken.name :
                        '';
                const symbol = metadataState.data.symbol ? trimString(metadataState.data.symbol) :
                    legacyToken ? legacyToken.symbol :
                        '';
                const icon = image ? image : legacyToken?.logoURI ? legacyToken.logoURI : '';
                return { addr, decimals, name, symbol, icon };
            }
            else {
                return undefined;
            }
        }
        catch (e) {
            return undefined;
        }
    }
    /**
     * List popular tokens.
     *
     * [USDT, USDC, SOL, RACE]
     */
    async listTokens() {
        const popularTokenAddrs = [
            "Es9vMFrzaCERmJfrF4H2FYD4KCoNkY11McCe8BenwNYB",
            "EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v",
            "So11111111111111111111111111111111111111112",
            "RACE5fnTKB9obGtCusArTQ6hhdNXAtf3HarvJM17rxJ"
        ];
        let tokens = [];
        for (const addr of popularTokenAddrs) {
            const tokenInfo = await this.getToken(addr);
            if (tokenInfo !== undefined) {
                tokens.push(tokenInfo);
            }
        }
        return tokens;
    }
    async fetchBalances(walletAddr, tokenAddrs) {
        const walletKey = new web3_js_1.PublicKey(walletAddr);
        let ret = new Map();
        for (const tokenAddr of tokenAddrs) {
            const tokenAccountKey = (0, spl_token_1.getAssociatedTokenAddressSync)(new web3_js_1.PublicKey(tokenAddr), walletKey);
            try {
                const resp = await __classPrivateFieldGet(this, _SolanaTransport_conn, "f").getTokenAccountBalance(tokenAccountKey);
                ret.set(tokenAddr, BigInt(resp.value.amount));
            }
            catch (e) {
                ret.set(tokenAddr, 0n);
            }
        }
        return ret;
    }
    async getNft(addr) {
        let mintKey;
        if (addr instanceof web3_js_1.PublicKey) {
            mintKey = addr;
        }
        else {
            mintKey = new web3_js_1.PublicKey(addr);
        }
        try {
            const mint = await (0, spl_token_1.getMint)(__classPrivateFieldGet(this, _SolanaTransport_conn, "f"), mintKey, 'finalized');
            // Non-zero decimals stands for a fungible token
            if (mint.decimals !== 0) {
                return undefined;
            }
            const [metadataKey] = web3_js_1.PublicKey.findProgramAddressSync([Buffer.from('metadata', 'utf8'), constants_2.METAPLEX_PROGRAM_ID.toBuffer(), mintKey.toBuffer()], constants_2.METAPLEX_PROGRAM_ID);
            const metadataAccount = await __classPrivateFieldGet(this, _SolanaTransport_conn, "f").getAccountInfo(metadataKey);
            let metadataState;
            if (metadataAccount !== null) {
                metadataState = metadata_1.Metadata.deserialize(metadataAccount.data);
            }
            if (metadataState !== undefined) {
                const image = await this._fetchImageFromDataUri(metadataState.data.uri);
                if (image === undefined)
                    return undefined;
                return {
                    addr: mint.address.toBase58(),
                    name: trimString(metadataState.data.name),
                    symbol: trimString(metadataState.data.symbol),
                    image,
                    collection: metadataState?.collection?.key.toBase58(),
                };
            }
            else {
                return undefined;
            }
        }
        catch (e) {
            console.warn(e);
            return undefined;
        }
    }
    async listNfts(walletAddr) {
        let nfts = [];
        const ownerKey = new web3_js_1.PublicKey(walletAddr);
        const parsedTokenAccounts = await __classPrivateFieldGet(this, _SolanaTransport_conn, "f").getParsedTokenAccountsByOwner(ownerKey, { programId: spl_token_1.TOKEN_PROGRAM_ID });
        for (const a of parsedTokenAccounts.value) {
            if (a.account.data.parsed.info.tokenAmount.amount !== '1'
                || a.account.data.parsed.info.tokenAmount.decimals !== 0) {
                continue;
            }
            const nft = await this.getNft(a.account.data.parsed.info.mint);
            if (nft !== undefined) {
                nfts.push(nft);
            }
        }
        return nfts;
    }
    async _getGameState(gameAccoutKey) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const gameAccount = await conn.getAccountInfo(gameAccoutKey);
        if (gameAccount !== null) {
            const data = gameAccount.data;
            return accounts_1.GameState.deserialize(data);
        }
        else {
            return undefined;
        }
    }
    async _getMultiGameStates(gameAccountKeys) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const accountsInfo = await conn.getMultipleAccountsInfo(gameAccountKeys);
        const ret = [];
        for (const accountInfo of accountsInfo) {
            if (accountInfo === null) {
                ret.push(undefined);
            }
            else {
                ret.push(accounts_1.GameState.deserialize(accountInfo.data));
            }
        }
        return ret;
    }
    async _getRegState(regKey) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const regAccount = await conn.getAccountInfo(regKey);
        if (regAccount !== null) {
            const data = regAccount.data;
            return accounts_1.RegistryState.deserialize(data);
        }
        else {
            return undefined;
        }
    }
    async _getServerState(serverKey) {
        const conn = __classPrivateFieldGet(this, _SolanaTransport_conn, "f");
        const profileKey = await web3_js_1.PublicKey.createWithSeed(serverKey, constants_1.SERVER_PROFILE_SEED, constants_2.PROGRAM_ID);
        const serverAccount = await conn.getAccountInfo(profileKey);
        if (serverAccount !== null) {
            const data = serverAccount.data;
            return accounts_1.ServerState.deserialize(data);
        }
        else {
            return undefined;
        }
    }
}
exports.SolanaTransport = SolanaTransport;
_SolanaTransport_conn = new WeakMap(), _SolanaTransport_legacyTokens = new WeakMap();
async function makeTransaction(conn, feePayer) {
    const { blockhash, lastValidBlockHeight } = await conn.getLatestBlockhash();
    return new web3_js_1.Transaction({
        feePayer,
        blockhash,
        lastValidBlockHeight,
    });
}
//# sourceMappingURL=solana-transport.js.map