var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { PublicKey, SYSVAR_RENT_PUBKEY, TransactionInstruction } from '@solana/web3.js';
import { TOKEN_PROGRAM_ID, getAssociatedTokenAddressSync } from '@solana/spl-token';
import { PROGRAM_ID, METAPLEX_PROGRAM_ID, PLAYER_PROFILE_SEED } from './constants';
import { field, serialize } from '@race-foundation/borsh';
import { Buffer } from 'buffer';
// Instruction types
export var Instruction;
(function (Instruction) {
    Instruction[Instruction["CreateGameAccount"] = 0] = "CreateGameAccount";
    Instruction[Instruction["CloseGameAccount"] = 1] = "CloseGameAccount";
    Instruction[Instruction["CreateRegistration"] = 2] = "CreateRegistration";
    Instruction[Instruction["CreatePlayerProfile"] = 3] = "CreatePlayerProfile";
    Instruction[Instruction["RegisterServer"] = 4] = "RegisterServer";
    Instruction[Instruction["Settle"] = 5] = "Settle";
    Instruction[Instruction["Vote"] = 6] = "Vote";
    Instruction[Instruction["ServeGame"] = 7] = "ServeGame";
    Instruction[Instruction["RegisterGame"] = 8] = "RegisterGame";
    Instruction[Instruction["UnregisterGame"] = 9] = "UnregisterGame";
    Instruction[Instruction["JoinGame"] = 10] = "JoinGame";
    Instruction[Instruction["PublishGame"] = 11] = "PublishGame";
})(Instruction || (Instruction = {}));
// Instruction data definitations
class Serialize {
    serialize() {
        return Buffer.from(serialize(this));
    }
}
export class CreatePlayerProfileData extends Serialize {
    constructor(nick) {
        super();
        this.instruction = Instruction.CreatePlayerProfile;
        this.nick = nick;
    }
}
__decorate([
    field('u8')
], CreatePlayerProfileData.prototype, "instruction", void 0);
__decorate([
    field('string')
], CreatePlayerProfileData.prototype, "nick", void 0);
export class CloseGameAccountData extends Serialize {
    constructor() {
        super();
        this.instruction = Instruction.CloseGameAccount;
    }
}
__decorate([
    field('u8')
], CloseGameAccountData.prototype, "instruction", void 0);
export class CreateGameAccountData extends Serialize {
    constructor(params) {
        super();
        this.instruction = Instruction.CreateGameAccount;
        this.title = '';
        this.maxPlayers = 0;
        this.minDeposit = 0n;
        this.maxDeposit = 0n;
        this.data = Uint8Array.from([]);
        Object.assign(this, params);
    }
}
__decorate([
    field('u8')
], CreateGameAccountData.prototype, "instruction", void 0);
__decorate([
    field('string')
], CreateGameAccountData.prototype, "title", void 0);
__decorate([
    field('u16')
], CreateGameAccountData.prototype, "maxPlayers", void 0);
__decorate([
    field('u64')
], CreateGameAccountData.prototype, "minDeposit", void 0);
__decorate([
    field('u64')
], CreateGameAccountData.prototype, "maxDeposit", void 0);
__decorate([
    field('u8-array')
], CreateGameAccountData.prototype, "data", void 0);
export class JoinGameData extends Serialize {
    constructor(amount, accessVersion, position, verifyKey) {
        super();
        this.instruction = Instruction.JoinGame;
        this.amount = amount;
        this.accessVersion = accessVersion;
        this.position = position;
        this.verifyKey = verifyKey;
    }
}
__decorate([
    field('u8')
], JoinGameData.prototype, "instruction", void 0);
__decorate([
    field('u64')
], JoinGameData.prototype, "amount", void 0);
__decorate([
    field('u64')
], JoinGameData.prototype, "accessVersion", void 0);
__decorate([
    field('u16')
], JoinGameData.prototype, "position", void 0);
__decorate([
    field('string')
], JoinGameData.prototype, "verifyKey", void 0);
export class PublishGameData extends Serialize {
    constructor(uri, name, symbol) {
        super();
        this.instruction = Instruction.PublishGame;
        this.uri = uri;
        this.name = name;
        this.symbol = symbol;
    }
}
__decorate([
    field('u8')
], PublishGameData.prototype, "instruction", void 0);
__decorate([
    field('string')
], PublishGameData.prototype, "uri", void 0);
__decorate([
    field('string')
], PublishGameData.prototype, "name", void 0);
__decorate([
    field('string')
], PublishGameData.prototype, "symbol", void 0);
// Instruction helpers
export function createPlayerProfile(ownerKey, profileKey, nick, pfpKey) {
    const data = new CreatePlayerProfileData(nick).serialize();
    return new TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: profileKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pfpKey || PublicKey.default,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: PROGRAM_ID,
        data: Buffer.from(data),
    });
}
export function createGameAccount(opts) {
    const data = new CreateGameAccountData(opts).serialize();
    return new TransactionInstruction({
        keys: [
            {
                pubkey: opts.ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: opts.gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: opts.stakeAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: opts.mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: opts.gameBundleKey,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: PROGRAM_ID,
        data,
    });
}
export function closeGameAccount(opts) {
    const { ownerKey, gameAccountKey, regAccountKey, gameStakeKey } = opts;
    const data = new CloseGameAccountData().serialize();
    let [pda, _] = PublicKey.findProgramAddressSync([gameAccountKey.toBuffer()], PROGRAM_ID);
    return new TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: regAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: gameStakeKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: PROGRAM_ID,
        data,
    });
}
export async function join(opts) {
    const { playerKey, paymentKey, gameAccountKey, mint, stakeAccountKey, amount, accessVersion, position, verifyKey } = opts;
    const profileKey = await PublicKey.createWithSeed(playerKey, PLAYER_PROFILE_SEED, PROGRAM_ID);
    let [pda, _] = PublicKey.findProgramAddressSync([gameAccountKey.toBuffer()], PROGRAM_ID);
    const data = new JoinGameData(amount, accessVersion, position, verifyKey).serialize();
    return new TransactionInstruction({
        keys: [
            {
                pubkey: playerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: profileKey,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: paymentKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: stakeAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pda,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: PROGRAM_ID,
        data,
    });
}
export function publishGame(opts) {
    const { ownerKey, mint, uri, name, symbol } = opts;
    let [metadataPda] = PublicKey.findProgramAddressSync([Buffer.from('metadata', 'utf8'), METAPLEX_PROGRAM_ID.toBuffer(), mint.toBuffer()], METAPLEX_PROGRAM_ID);
    let [editonPda] = PublicKey.findProgramAddressSync([Buffer.from('metadata', 'utf8'), METAPLEX_PROGRAM_ID.toBuffer(), mint.toBuffer(), Buffer.from('edition', 'utf8')], METAPLEX_PROGRAM_ID);
    let ata = getAssociatedTokenAddressSync(mint, ownerKey);
    let data = new PublishGameData(uri, name, symbol).serialize();
    return new TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: ata,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: metadataPda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: editonPda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: METAPLEX_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: PublicKey.default,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: PROGRAM_ID,
        data,
    });
}
//# sourceMappingURL=instruction.js.map